@@ -4,11 +4,11 @@ top-level domain) from the registered domain and subdomains of a URL.
 
     >>> import tldextract
     >>> tldextract.extract('http://forums.news.cnn.com/')
-    ExtractResult(subdomain='forums.news', domain='cnn', tld='com')
+    ExtractResult(subdomain='forums.news', domain='cnn', suffix='com')
     >>> tldextract.extract('http://forums.bbc.co.uk/') # United Kingdom
-    ExtractResult(subdomain='forums', domain='bbc', tld='co.uk')
+    ExtractResult(subdomain='forums', domain='bbc', suffix='co.uk')
     >>> tldextract.extract('http://www.worldbank.org.kg/') # Kyrgyzstan
-    ExtractResult(subdomain='www', domain='worldbank', tld='org.kg')
+    ExtractResult(subdomain='www', domain='worldbank', suffix='org.kg')
 
 `ExtractResult` is a namedtuple, so it's simple to access the parts you want.
 
@@ -30,6 +30,7 @@ import logging
 from operator import itemgetter
 import os
 import sys
+import warnings
 
 try:
     import pkg_resources
@@ -46,22 +47,30 @@ except ImportError:
 
 import re
 import socket
-import urllib2
-import urlparse
+try: # pragma: no cover
+    # Python 2
+    from urllib2 import urlopen, URLError
+    from urlparse import scheme_chars
+except ImportError: # pragma: no cover
+    # Python 3
+    from urllib.request import urlopen
+    from urllib.error import URLError
+    from urllib.parse import scheme_chars
+    unicode = str
 
 LOG = logging.getLogger("tldextract")
 
-SCHEME_RE = re.compile(r'^([' + urlparse.scheme_chars + ']+:)?//')
+SCHEME_RE = re.compile(r'^([' + scheme_chars + ']+:)?//')
 IP_RE = re.compile(r'^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$')
 
 class ExtractResult(tuple):
-    'ExtractResult(subdomain, domain, tld)'
+    'ExtractResult(subdomain, domain, suffix)'
     __slots__ = ()
-    _fields = ('subdomain', 'domain', 'tld')
+    _fields = ('subdomain', 'domain', 'suffix')
 
-    def __new__(_cls, subdomain, domain, tld):
-        'Create new instance of ExtractResult(subdomain, domain, tld)'
-        return tuple.__new__(_cls, (subdomain, domain, tld))
+    def __new__(_cls, subdomain, domain, suffix):
+        'Create new instance of ExtractResult(subdomain, domain, suffix)'
+        return tuple.__new__(_cls, (subdomain, domain, suffix))
 
     @classmethod
     def _make(cls, iterable, new=tuple.__new__, len=len):
@@ -73,15 +82,17 @@ class ExtractResult(tuple):
 
     def __repr__(self):
         'Return a nicely formatted representation string'
-        return 'ExtractResult(subdomain=%r, domain=%r, tld=%r)' % self
+        return 'ExtractResult(subdomain=%r, domain=%r, suffix=%r)' % self
 
     def _asdict(self):
         'Return a new dict which maps field names to their values'
-        return dict(zip(self._fields, self))
+        base_zip = zip(self._fields, self)
+        zipped = base_zip + [('tld', self.tld)]
+        return dict(zipped)
 
     def _replace(_self, **kwds):
         'Return a new ExtractResult object replacing specified fields with new values'
-        result = _self._make(map(kwds.pop, ('subdomain', 'domain', 'tld'), _self))
+        result = _self._make(map(kwds.pop, ('subdomain', 'domain', 'suffix'), _self))
         if kwds:
             raise ValueError('Got unexpected field names: %r' % kwds.keys())
         return result
@@ -92,7 +103,26 @@ class ExtractResult(tuple):
 
     subdomain = property(itemgetter(0), doc='Alias for field number 0')
     domain = property(itemgetter(1), doc='Alias for field number 1')
-    tld = property(itemgetter(2), doc='Alias for field number 2')
+    suffix = property(itemgetter(2), doc='Alias for field number 2')
+
+    @property
+    def tld(self):
+      warnings.warn('This use of tld is misleading. Use `suffix` instead.', DeprecationWarning)
+      return self.suffix
+
+    @property
+    def registered_domain(self):
+      """
+      Joins the domain and suffix fields with a dot, if they're both set.
+
+      >>> extract('http://forums.bbc.co.uk').registered_domain
+      'bbc.co.uk'
+      >>> extract('http://localhost:8080').registered_domain
+      ''
+      """
+      if self.domain and self.suffix:
+          return self.domain + '.' + self.suffix
+      return ''
 
 class TLDExtract(object):
     def __init__(self, fetch=True, cache_file=''):
@@ -110,7 +140,9 @@ class TLDExtract(object):
 
         """
         self.fetch = fetch
-        self.cache_file = cache_file or os.path.join(os.path.dirname(__file__), '.tld_set')
+        self.cache_file = os.path.expanduser(cache_file or
+            os.environ.get("TLDEXTRACT_CACHE",
+                os.path.join(os.path.dirname(__file__), '.tld_set')))
         self._extractor = None
 
     def __call__(self, url):
@@ -120,16 +152,17 @@ class TLDExtract(object):
 
         >>> extract = TLDExtract()
         >>> extract('http://forums.news.cnn.com/')
-        ExtractResult(subdomain='forums.news', domain='cnn', tld='com')
+        ExtractResult(subdomain='forums.news', domain='cnn', suffix='com')
         >>> extract('http://forums.bbc.co.uk/')
-        ExtractResult(subdomain='forums', domain='bbc', tld='co.uk')
+        ExtractResult(subdomain='forums', domain='bbc', suffix='co.uk')
         """
         netloc = SCHEME_RE.sub("", url) \
           .partition("/")[0] \
           .partition("?")[0] \
           .partition("#")[0] \
           .split("@")[-1] \
-          .partition(":")[0]
+          .partition(":")[0] \
+          .rstrip(".")
 
         registered_domain, tld = self._get_tld_extractor().extract(netloc)
         if not tld and netloc and netloc[0].isdigit():
@@ -145,6 +178,13 @@ class TLDExtract(object):
         subdomain, _, domain = registered_domain.rpartition('.')
         return ExtractResult(subdomain, domain, tld)
 
+    def update(self, fetch_now=False):
+        if os.path.exists(self.cache_file):
+            os.unlink(self.cache_file)
+        self._extractor = None
+        if fetch_now:
+            self._get_tld_extractor()
+
     def _get_tld_extractor(self):
         if self._extractor:
             return self._extractor
@@ -154,11 +194,11 @@ class TLDExtract(object):
             with open(cached_file) as f:
                 self._extractor = _PublicSuffixListTLDExtractor(pickle.load(f))
                 return self._extractor
-        except IOError, ioe:
+        except IOError as ioe:
             file_not_found = ioe.errno == errno.ENOENT
             if not file_not_found:
               LOG.error("error reading TLD cache file %s: %s", cached_file, ioe)
-        except Exception, ex:
+        except Exception as ex:
             LOG.error("error reading TLD cache file %s: %s", cached_file, ex)
 
         tlds = frozenset()
@@ -178,12 +218,15 @@ class TLDExtract(object):
                 snapshot = sorted(pickle.load(snapshot_file))
             new = sorted(tlds)
             for line in difflib.unified_diff(snapshot, new, fromfile=".tld_set_snapshot", tofile=cached_file):
-                print >> sys.stderr, line.encode('utf-8')
+                if sys.version_info < (3,):
+                    sys.stderr.write(line.encode('utf-8') + "\n")
+                else:
+                    sys.stderr.write(line + "\n")
 
         try:
             with open(cached_file, 'wb') as f:
                 pickle.dump(tlds, f)
-        except IOError, e:
+        except IOError as e:
             LOG.warn("unable to cache TLDs in file %s: %s", cached_file, e)
 
         self._extractor = _PublicSuffixListTLDExtractor(tlds)
@@ -195,10 +238,14 @@ TLD_EXTRACTOR = TLDExtract()
 def extract(url):
     return TLD_EXTRACTOR(url)
 
+@wraps(TLD_EXTRACTOR.update)
+def update(*args, **kwargs):
+    return TLD_EXTRACTOR.update(*args, **kwargs)
+
 def _fetch_page(url):
     try:
-        return unicode(urllib2.urlopen(url).read(), 'utf-8')
-    except urllib2.URLError, e:
+        return unicode(urlopen(url).read(), 'utf-8')
+    except URLError as e:
         LOG.error(e)
         return u''
 
@@ -215,18 +262,56 @@ class _PublicSuffixListTLDExtractor(object):
 
     def extract(self, netloc):
         spl = netloc.split('.')
+        lower_spl = tuple(el.lower() for el in spl)
         for i in range(len(spl)):
-            maybe_tld = '.'.join(spl[i:])
+            maybe_tld = '.'.join(lower_spl[i:])
             exception_tld = '!' + maybe_tld
             if exception_tld in self.tlds:
                 return '.'.join(spl[:i+1]), '.'.join(spl[i+1:])
 
-            wildcard_tld = '*.' + '.'.join(spl[i+1:])
-            if wildcard_tld in self.tlds or maybe_tld in self.tlds:
-                return '.'.join(spl[:i]), maybe_tld
+            if maybe_tld in self.tlds:
+                return '.'.join(spl[:i]), '.'.join(spl[i:])
+
+            wildcard_tld = '*.' + '.'.join(lower_spl[i+1:])
+            if wildcard_tld in self.tlds:
+                return '.'.join(spl[:i]), '.'.join(spl[i:])
 
         return netloc, ''
 
+
+def main():
+    """docstring for main"""
+    import argparse
+
+    distribution = pkg_resources.get_distribution('tldextract')
+
+    parser = argparse.ArgumentParser(
+        version='%(prog)s ' + distribution.version,
+        description='Parse hostname from a url or fqdn')
+
+    parser.add_argument('input', metavar='fqdn|url',
+                        type=unicode, nargs='*', help='fqdn or url')
+
+    parser.add_argument('-u', '--update', default=False, action='store_true', help='force fetch the latest TLD definitions')
+    parser.add_argument('-c', '--cache_file', help='use an alternate TLD definition file')
+
+    args = parser.parse_args()
+
+    if args.cache_file:
+        TLD_EXTRACTOR.cache_file = args.cache_file
+
+    if args.update:
+        try:
+            TLD_EXTRACTOR.update(True)
+        except Exception as exc:
+            sys.stderr.write(exc + "\n")
+            exit(2)
+    elif len(args.input) is 0:
+        parser.print_usage()
+        exit(1)
+
+    for i in args.input:
+        print(' '.join(extract(i)))
+
 if __name__ == "__main__":
-    url = sys.argv[1]
-    print ' '.join(extract(url))
+    main()
